@@ -2,14 +2,16 @@
 
 from __future__ import annotations
 
-import decimal
-from collections.abc import MutableMapping
+from logging import Logger
 from typing import Any, Iterable
 
 import jsonschema.exceptions as jsonschema_exceptions
 import simplejson as json
 import sqlalchemy
 from pendulum import now
+from singer_sdk.helpers._typing import (
+    DatetimeErrorTreatmentEnum,
+)
 from singer_sdk.sinks import SQLSink
 from sqlalchemy.sql.expression import bindparam
 
@@ -52,6 +54,11 @@ class ClickhouseSink(SQLSink):
             db_name=self.database_name,
         )
 
+    @property
+    def datetime_error_treatment(self) -> DatetimeErrorTreatmentEnum:
+        """Return a treatment to use for datetime parse errors: ERROR. MAX, or NULL."""
+        return DatetimeErrorTreatmentEnum.NULL
+
     def bulk_insert_records(
             self,
             full_table_name: str,
@@ -147,7 +154,7 @@ class ClickhouseSink(SQLSink):
             Validated record.
         """
         # Pre-validate and correct string type mismatches.
-        record = self._pre_validate_for_string_type(record)
+        record = pre_validate_for_string_type(record, self.schema, self.logger)
 
         try:
             self._validator.validate(record)
@@ -163,51 +170,59 @@ class ClickhouseSink(SQLSink):
 
         return record
 
-    def _pre_validate_for_string_type(self, record: dict) -> dict:
-        """Pre-validate record for string type mismatches and correct them.
+def pre_validate_for_string_type(
+    record: dict,
+    schema: dict,
+    logger: Logger | None = None,
+) -> dict:
+    """Pre-validate record for string type mismatches and correct them.
 
-        Args:
-            record: Individual record in the stream.
+    Args:
+        record: Individual record in the stream.
+        schema: JSON schema for the stream.
+        logger: Logger to use for logging.
 
-        Returns:
-            Record with corrected string type mismatches.
-        """
-        for key, value in record.items():
-            # Checking if the schema expects a string for this key.
-            expected_type = self.schema.get("properties", {}).get(key, {}).get("type")
-            if expected_type is None:
-                continue
-            if not isinstance(expected_type, list):
-                expected_type = [expected_type]
-            if "string" in expected_type and not isinstance(value, str):
-                # Convert the value to string if it's not already a string.
-                record[key] = (
-                    json.dumps(record[key])
-                    if isinstance(value, (dict, list)) else str(value)
-                )
-                if self.logger:
-                    self.logger.debug(
-                        f"Converted field {key} to string: {record[key]}",
+    Returns:
+        Record with corrected string type mismatches.
+    """
+    if schema is None:
+        if logger:
+            logger.debug("Schema is None, skipping pre-validation.")
+        return record
+
+    for key, value in record.items():
+        # Checking if the schema expects a string for this key.
+        key_properties = schema.get("properties", {}).get(key, {})
+        expected_type = key_properties.get("type")
+        if expected_type is None:
+            continue
+        if not isinstance(expected_type, list):
+            expected_type = [expected_type]
+
+        if "object" in expected_type and isinstance(value, dict):
+            pre_validate_for_string_type(
+                value,
+                schema.get("properties", {}).get(key),
+                logger,
+            )
+        elif "array" in expected_type and isinstance(value, list):
+            items_schema = key_properties.get("items")
+            for i, item in enumerate(value):
+                if items_schema["type"] == "object" and isinstance(item, dict):
+                    value[i] = pre_validate_for_string_type(
+                        item,
+                        key_properties.get("items"),
+                        logger,
                     )
+        elif "string" in expected_type and not isinstance(value, str):
+            # Convert the value to string if it's not already a string.
+            record[key] = (
+                json.dumps(record[key])
+                if isinstance(value, (dict, list)) else str(value)
+            )
+            if logger:
+                logger.debug(
+                    f"Converted field {key} to string: {record[key]}",
+                )
 
-        return self._convert_decimal_to_float(record)
-
-    def _convert_decimal_to_float(self, obj):
-        """Recursively convert all Decimal values in a dictionary to floats.
-
-        Args:
-        obj: The input object (dictionary, list, or any other data type).
-
-        Returns:
-        The object with all Decimal values converted to strings.
-        """
-        if isinstance(obj, MutableMapping):
-            for key, value in obj.items():
-                obj[key] = self._convert_decimal_to_float(value)
-        elif isinstance(obj, list):
-            for i, item in enumerate(obj):
-                obj[i] = self._convert_decimal_to_float(item)
-        elif isinstance(obj, decimal.Decimal):
-            return float(obj)
-
-        return obj
+    return record
